import {
  FIRST_ARRAY_ELEMENT,
  SECOND_ARRAY_ELEMENT,
  SIZE_OF_EMPTY_ARRAY,
  THIRD_ARRAY_ELEMENT,
} from '@/constants/common';
import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { ELEMENT_SEARCH_RESULT_MOCK_ALIAS } from '@/models/mocks/elementSearchResultMock';
import { apiPath } from '@/redux/apiPath';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { waitFor } from '@testing-library/react';
import { HttpStatusCode } from 'axios';
import { handleAliasResults } from './handleAliasResults';

const mockedAxiosOldClient = mockNetworkResponse();

describe('handleAliasResults - util', () => {
  const { store } = getReduxStoreWithActionsListener();
  const { dispatch } = store;

  mockedAxiosOldClient
    .onGet(
      apiPath.getBioEntityContentsStringWithQuery({
        searchQuery: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id.toString(),
        isPerfectMatch: true,
      }),
    )
    .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

  beforeAll(async () => {
    handleAliasResults(
      dispatch,
      ELEMENT_SEARCH_RESULT_MOCK_ALIAS,
    )(ELEMENT_SEARCH_RESULT_MOCK_ALIAS);
  });

  it('should run selectTab as first action', async () => {
    await waitFor(() => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[FIRST_ARRAY_ELEMENT].type).toEqual('drawer/selectTab');
    });
  });

  it('should run openBioEntityDrawerById as second action', async () => {
    await waitFor(() => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[SECOND_ARRAY_ELEMENT].type).toEqual('drawer/openBioEntityDrawerById');
    });
  });

  it('should run getMultiBioEntity as third action', async () => {
    await waitFor(() => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[THIRD_ARRAY_ELEMENT].type).toEqual('project/getMultiBioEntity/pending');
    });
  });
});
