/* eslint-disable no-magic-numbers */
import { FIRST_ARRAY_ELEMENT, SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { reactionsFixture } from '@/models/fixtures/reactionFixture';
import {
  ELEMENT_SEARCH_RESULT_MOCK_ALIAS,
  ELEMENT_SEARCH_RESULT_MOCK_REACTION,
} from '@/models/mocks/elementSearchResultMock';
import { apiPath } from '@/redux/apiPath';
import { INITIAL_STORE_STATE_MOCK } from '@/redux/root/root.fixtures';
import { mockNetworkNewAPIResponse, mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { HttpStatusCode } from 'axios';
import { handleReactionResults } from './handleReactionResults';

const mockedAxiosOldClient = mockNetworkResponse();
const mockedAxiosNewClient = mockNetworkNewAPIResponse();

describe('handleReactionResults - util', () => {
  const { store } = getReduxStoreWithActionsListener({
    ...INITIAL_STORE_STATE_MOCK,
  });
  const { dispatch } = store;

  mockedAxiosNewClient
    .onGet(
      apiPath.getBioEntityContentsStringWithQuery({
        searchQuery: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id.toString(),
        isPerfectMatch: true,
      }),
    )
    .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

  mockedAxiosOldClient
    .onGet(apiPath.getReactionsWithIds([ELEMENT_SEARCH_RESULT_MOCK_REACTION.id]))
    .reply(HttpStatusCode.Ok, [
      {
        ...reactionsFixture[0],
        reactants: [],
        products: [],
        modifiers: [
          {
            ...reactionsFixture[0].modifiers[0],
            aliasId: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id,
          },
        ],
      },
    ]);

  beforeEach(async () => {
    await handleReactionResults(
      dispatch,
      ELEMENT_SEARCH_RESULT_MOCK_REACTION,
    )(ELEMENT_SEARCH_RESULT_MOCK_REACTION);
  });

  it('should run getReactionsByIds as first action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[0].type).toEqual('reactions/getByIds/pending');
    expect(actions[1].type).toEqual('reactions/getByIds/fulfilled');
  });

  it('should run openReactionDrawerById to empty array as second action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[2].type).toEqual('drawer/openReactionDrawerById');
    expect(actions[2].payload).toEqual(reactionsFixture[FIRST_ARRAY_ELEMENT].id);
  });

  it('should run setBioEntityContent to empty array as third action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[3].type).toEqual('project/getMultiBioEntity/pending');
  });

  it('should run getBioEntityContents as fourth action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[4].type).toEqual('project/getBioEntityContents/pending');
  });

  it('should run getBioEntityContents fullfilled as fifth action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[5].type).toEqual('project/getBioEntityContents/fulfilled');
  });

  it('should run addNumbersToEntityNumberData as sixth action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[6].type).toEqual('entityNumber/addNumbersToEntityNumberData');
  });

  it('should run getByIds as seventh action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[7].type).toEqual('reactions/getByIds/pending');
  });
});
