import { FIRST_ARRAY_ELEMENT, SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { getMultiBioEntity } from '@/redux/bioEntity/bioEntity.thunks';
import { openReactionDrawerById } from '@/redux/drawer/drawer.slice';
import { getReactionsByIds } from '@/redux/reactions/reactions.thunks';
import { AppDispatch } from '@/redux/store';
import { searchFitBounds } from '@/services/pluginsManager/map/triggerSearch/searchFitBounds';
import { PluginsEventBus } from '@/services/pluginsManager/pluginsEventBus';
import { ElementSearchResult } from '@/types/models';
import { getBioEntitiesIdsFromReaction } from './getBioEntitiesIdsFromReaction';

/* prettier-ignore */
export const handleReactionResults =
  (dispatch: AppDispatch, closestSearchResult: ElementSearchResult, hasFitBounds?: boolean, fitBoundsZoom?: number) =>
    async ({ id }: ElementSearchResult): Promise<void> => {
      const data = await dispatch(getReactionsByIds([id]));
      const payload = data?.payload;
      if (!data || !payload || typeof payload === "string" || payload.data.length === SIZE_OF_EMPTY_ARRAY) {
        return;
      }

      const reaction = payload.data[FIRST_ARRAY_ELEMENT];
      const bioEntitiesIds = getBioEntitiesIdsFromReaction(reaction);

      dispatch(openReactionDrawerById(reaction.id));
      await dispatch(
        getMultiBioEntity({
          searchQueries: bioEntitiesIds,
          isPerfectMatch: true },
        )
      ).unwrap().then((bioEntityContents) => {
        PluginsEventBus.dispatchEvent('onSearch', {
          type: 'bioEntity',
          searchValues: [closestSearchResult],
          results: [bioEntityContents],
        });

        if (hasFitBounds) {
          searchFitBounds(fitBoundsZoom);
        }
      });
    };
